#include <stdio.h>
#include <glib.h>
#include "datasource.h"
#include "mh-cpu-source.h"

typedef struct _MhCpuSourceData MhCpuSourceData;
struct _MhCpuSourceData {
	guint prev;
	guint prev_idle;
};

static void *mh_cpu_source_init(DataSource *data_source);
static double mh_cpu_source_get_value(void *ds_data);
static void mh_cpu_source_deinit(void *ds_data);

void mh_cpu_source_load(DataSource *data_source)
{
	data_source->init = mh_cpu_source_init;
	data_source->get_value = mh_cpu_source_get_value;
	data_source->deinit = mh_cpu_source_deinit;

	data_source->name = "CPU load";
	data_source->max_text = "<b>100%</b>";
	data_source->half_text = "<b>50%</b>";
	data_source->min_text = "<b>0%</b>";
}

static void *mh_cpu_source_init(DataSource *data_source)
{
	MhCpuSourceData *data = g_new0(MhCpuSourceData, 1);

	(void)data_source;

	data->prev = 0;

	return data;
}

static double mh_cpu_source_get_value(void *ds_data)
{
	MhCpuSourceData *data = ds_data;
	guint user, nice, kern, idle, change, change_idle;
	FILE *proc_stat = fopen("/proc/stat", "r");
	fscanf(proc_stat, "cpu %u %u %u %u", &user, &nice, &kern, &idle);
	fclose(proc_stat);

	change = user + nice + kern + idle - data->prev;
	data->prev = user + nice + kern + idle;
	change_idle = idle - data->prev_idle;
	data->prev_idle = idle;

	return (double)(change - change_idle) / change;
}

static void mh_cpu_source_deinit(void *ds_data)
{
	g_free(ds_data);
}
