#include <stdio.h>
#include <glib.h>
#include "datasource.h"
#include "mh-mem-source.h"

typedef struct _MhMemSourceData MhMemSourceData;
struct _MhMemSourceData {
	gchar *max;
	gchar *half;
	gchar *min;
};

static void *mh_mem_source_init(DataSource *data_source);
static double mh_mem_source_get_value(void *ds_data);
static void mh_mem_source_deinit(void *ds_data);

void mh_mem_source_load(DataSource *data_source)
{
	data_source->init = mh_mem_source_init;
	data_source->get_value = mh_mem_source_get_value;
	data_source->deinit = mh_mem_source_deinit;

	data_source->name = "MEM use";
	data_source->min_text = "<b>0 MiB</b>";
}

static void *mh_mem_source_init(DataSource *data_source)
{
	MhMemSourceData *data = g_new0(MhMemSourceData, 1);

	FILE *meminfo = fopen("/proc/meminfo", "r");
	guint total;

	fscanf(meminfo,
			"MemTotal: %d kB", &total);
	fclose(meminfo);

	data->max = data_source->max_text = g_strdup_printf("<b>%d MiB</b>", total >> 10);
	data->half = data_source->half_text = g_strdup_printf("<b>%d MiB</b>", total >> 11);

	return data;
}

static double mh_mem_source_get_value(void *ds_data)
{
	FILE *meminfo = fopen("/proc/meminfo", "r");
	guint total, free, cached;

	(void)ds_data;

	fscanf(meminfo,
			"MemTotal: %d kB MemFree: %d kB "
			"Buffers: %*d kB Cached: %d kB",
			&total, &free, &cached);
	fclose(meminfo);

	return (double)(total - free - cached) / total;
}

static void mh_mem_source_deinit(void *ds_data)
{
	g_free(ds_data);
}
